% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rxp_pipeline.R
\name{flatten_derivations}
\alias{flatten_derivations}
\title{Flatten a list of derivations and pipelines}
\usage{
flatten_derivations(derivs)
}
\arguments{
\item{derivs}{A list that may contain \code{rxp_derivation} objects,
\code{rxp_pipeline} objects, or a mix of both.}
}
\value{
A flat list of \code{rxp_derivation} objects.
}
\description{
Takes a potentially nested list containing both \code{rxp_derivation} objects
and \code{rxp_pipeline} objects and returns a flat list of derivations with
metadata preserved.
}
\keyword{internal}
